/**
 * Copyright (c) 2014-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.extension.registry.overlay;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.util.HashSet;
import java.util.Properties;
import java.util.Set;

import org.easymock.EasyMock;
import org.easymock.IAnswer;
import org.junit.Test;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.configuration.ContainerConfiguration;
import org.ow2.petals.microkernel.api.configuration.DomainConfiguration;
import org.ow2.petals.microkernel.api.configuration.exception.ConfigurationException;
import org.ow2.petals.microkernel.api.extension.PetalsPreExtensionController;
import org.ow2.petals.microkernel.api.extension.exception.PetalsExtensionException;
import org.ow2.petals.microkernel.configuration.ConfigurationTestUtils;
import org.ow2.petals.microkernel.extension.PreExtensionControllerTestcase;
import org.ow2.petals.topology.generated.Container;
import org.ow2.petals.topology.generated.JmxService;
import org.ow2.petals.topology.generated.Topology;
import org.ow2.petals.topology.generated.TransportService;

/**
 * Unit tests of
 * {@link ExtensionController#isActivated(org.ow2.petals.microkernel.api.configuration.ConfigurationService)}.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class ExtensionController_IsActivatedTest extends PreExtensionControllerTestcase {

    private final static String CONTAINER_NAME_1 = "sample-1";

    private final static String CONTAINER_DESCRIPTION_1 = "description of the container 1";

    private final static int CONTAINER_TCP_PORT_1 = 7801;

    /**
     * <p>
     * Check {@link ExtensionController#isActivated(org.ow2.petals.microkernel.api.configuration.ConfigurationService)}
     * when:
     * <ul>
     * <li>the topology contains more than one container</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: the extension is not activated
     * </p>
     */
    @Test
    public void testMoreThanContainer() throws ConfigurationException, PetalsExtensionException {

        final Topology topology = this.getBasicSampleTopology();

        // We add another container to the basic sample topology
        final Container container_1 = new Container();
        topology.getDomain().getContainer().add(container_1);
        container_1.setName(CONTAINER_NAME_1);
        container_1.setDescription(CONTAINER_DESCRIPTION_1);
        container_1.setHost(ConfigurationTestUtils.CONTAINER_HOST);
        container_1.setUser(ConfigurationTestUtils.CONTAINER_JMX_USER);
        container_1.setPassword(ConfigurationTestUtils.CONTAINER_JMX_PWD);

        final JmxService jmxService0 = new JmxService();
        container_1.setJmxService(jmxService0);

        final TransportService transportService0 = new TransportService();
        transportService0.setTcpPort(CONTAINER_TCP_PORT_1);
        container_1.setTransportService(transportService0);

        final ConfigurationService configurationServiceMock = this.createConfigurationServiceMock(topology,
                new Properties());

        final PetalsPreExtensionController extensionCtrl = new ExtensionController();
        assertFalse("Extension is activated", extensionCtrl.isActivated(configurationServiceMock));

    }

    /**
     * <p>
     * Check {@link ExtensionController#isActivated(org.ow2.petals.microkernel.api.configuration.ConfigurationService)}
     * when:
     * <ul>
     * <li>the topology is the sample one, with only one container,</li>
     * <li>the property '{@link EmbeddedRegistryOverlayService#ACTIVATION_PROPERTY_NAME}' is not set</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: the extension is activated
     * </p>
     */
    @Test
    public void testSampleTopologyAndActivationPropertyNotSet() throws ConfigurationException, PetalsExtensionException {

        final Topology topology = this.getBasicSampleTopology();

        final ConfigurationService configurationServiceMock = this.createConfigurationServiceMock(topology,
                new Properties());

        final PetalsPreExtensionController extensionCtrl = new ExtensionController();
        assertTrue("Extension is not activated", extensionCtrl.isActivated(configurationServiceMock));

    }

    /**
     * <p>
     * Check {@link ExtensionController#isActivated(org.ow2.petals.microkernel.api.configuration.ConfigurationService)}
     * when:
     * <ul>
     * <li>the topology is the sample one, with only one container,</li>
     * <li>a registry implementation is set</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: the extension is not activated
     * </p>
     */
    @Test
    public void testSampleTopologyAndRegistryImplementationSet() throws ConfigurationException,
            PetalsExtensionException {

        final Topology topology = this.getBasicSampleTopology();
        topology.getDomain().setRegistryImplementation("my-own-registry-implementation");

        final ConfigurationService configurationServiceMock = this.createConfigurationServiceMock(topology,
                new Properties());

        final PetalsPreExtensionController extensionCtrl = new ExtensionController();
        assertFalse("Extension is activated", extensionCtrl.isActivated(configurationServiceMock));

    }

    /**
     * <p>
     * Check {@link ExtensionController#isActivated(org.ow2.petals.microkernel.api.configuration.ConfigurationService)}
     * when:
     * <ul>
     * <li>the topology is the sample one, with only one container,</li>
     * <li>the property '{@link EmbeddedRegistryOverlayService#ACTIVATION_PROPERTY_NAME}' is set to <code>false</code></li>
     * </ul>
     * </p>
     * <p>
     * Expected results: the extension is not activated
     * </p>
     */
    @Test
    public void testSampleTopologyAndActivationPropertySetToFalse() throws ConfigurationException,
            PetalsExtensionException {

        final Topology topology = this.getBasicSampleTopology();
        final Properties serverProps = new Properties();
        serverProps.setProperty(EmbeddedRegistryOverlayService.ACTIVATION_PROPERTY_NAME, Boolean.FALSE.toString());

        final ConfigurationService configurationServiceMock = this
                .createConfigurationServiceMock(topology, serverProps);

        final PetalsPreExtensionController extensionCtrl = new ExtensionController();
        assertFalse("Extension is activated", extensionCtrl.isActivated(configurationServiceMock));

    }

    /**
     * <p>
     * Check {@link ExtensionController#isActivated(org.ow2.petals.microkernel.api.configuration.ConfigurationService)}
     * when:
     * <ul>
     * <li>the topology is the sample one, with only one container,</li>
     * <li>the property '{@link EmbeddedRegistryOverlayService#ACTIVATION_PROPERTY_NAME}' is set to <code>true</code></li>
     * </ul>
     * </p>
     * <p>
     * Expected results: the extension is activated
     * </p>
     */
    @Test
    public void testSampleTopologyAndActivationPropertySetToTrue() throws ConfigurationException,
            PetalsExtensionException {

        final Topology topology = this.getBasicSampleTopology();
        final Properties serverProps = new Properties();
        serverProps.setProperty(EmbeddedRegistryOverlayService.ACTIVATION_PROPERTY_NAME, Boolean.TRUE.toString());

        final ConfigurationService configurationServiceMock = this
                .createConfigurationServiceMock(topology, serverProps);

        final PetalsPreExtensionController extensionCtrl = new ExtensionController();
        assertTrue("Extension is not activated", extensionCtrl.isActivated(configurationServiceMock));

    }

    /**
     * Create the configuration service mock used by the extension controller
     * 
     * @param topology
     *            The topology to used. Can not be <code>null</code>.
     * @param serverProps
     *            The server properties. Can not be <code>null</code>.
     * @return The configuration service mock
     * @throws ConfigurationException
     */
    private ConfigurationService createConfigurationServiceMock(final Topology topology, final Properties serverProps)
            throws ConfigurationException {

        assertNotNull("Topology is null", topology);
        assertNotNull("Topology has no domain", topology.getDomain());
        assertNotNull("Local server properties is null", serverProps);

        final DomainConfiguration domainConfiguration = new DomainConfiguration();
        domainConfiguration.setSharedAreaImplementation(topology.getDomain().getRegistryImplementation());

        final ConfigurationService configurationServiceMock = EasyMock.createMock(ConfigurationService.class);
        EasyMock.expect(configurationServiceMock.getLocalTopology()).andReturn(topology).anyTimes();
        EasyMock.expect(configurationServiceMock.getDomainConfiguration()).andReturn(domainConfiguration)
                .anyTimes();
        EasyMock.expect(configurationServiceMock.getContainersConfiguration())
                .andAnswer(new IAnswer<Set<ContainerConfiguration>>() {
                    @Override
                    public Set<ContainerConfiguration> answer() throws Throwable {
                        // Only the number of container configuration is used by ExtensionController.isActivated()
                        final Set<ContainerConfiguration> result = new HashSet<ContainerConfiguration>();
                        for (final Container container : topology.getDomain().getContainer()) {
                            final ContainerConfiguration containerCfg = new ContainerConfiguration();
                            containerCfg.setName(container.getName());
                            result.add(containerCfg);
                        }
                        return result;
                    }
                }).anyTimes();
        EasyMock.expect(configurationServiceMock.getServerProperties()).andReturn(serverProps);
        EasyMock.replay(configurationServiceMock);

        return configurationServiceMock;
    }
}
